namespace Refactory.Tests {
	using System;
	using Refactory;
	using NUnit.Framework;
	
	[TestFixture] public class LargeIntegerTest {		
		
		[Test] public void IntegerConstructor() {
			LargeInteger two = new LargeInteger(2);
			Assertion.Assert("1 NonNegative",LargeInteger.One.NonNegative);
			Assertion.Assert("!(1 NonPositive)",!LargeInteger.One.NonPositive);
			Assertion.Assert("!(-1 NonNegative)",!(new LargeInteger(-1).NonNegative));
			Assertion.Assert("-1 NonPositive",!LargeInteger.One.NonPositive);
			Assertion.Assert("0 NonNegative & 0 NonPositive",(new LargeInteger(0)).NonNegative && (new LargeInteger(0)).NonPositive);
			Assertion.Assert("1 != 2",LargeInteger.One != two);
			Assertion.Assert("!(1 == 2)",!(LargeInteger.One == two));
			Assertion.Assert("-1 << 31",new LargeInteger(-1 << 31) == new LargeInteger(-0x80000000L));
		}
		
		[Test] public void StringConstructor() {
			Assertion.Assert("12345678901234567890",new LargeInteger(12345678901234567890UL) == new LargeInteger("12345678901234567890"));
			Assertion.Assert("-54321",new LargeInteger(-54321) == new LargeInteger("-54321"));
			
			LargeInteger l;
			try {
				l = new LargeInteger("1A93");
				Assertion.Assert("Invalid radix",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("--1");
				Assertion.Assert("--1",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("");
				Assertion.Assert("",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("1 2");
				Assertion.Assert("1 2",false);
			} catch (ArgumentException) {
			}
		}
		
		[Test] public void StringRadixConstructor() {
			Assertion.Assert("0xFEDCBA98",new LargeInteger(0xFEDCBA98U) == new LargeInteger("FEDCBA98",16));
			Assertion.Assert("36rABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",new LargeInteger("13E4283D679DEDEEA7B5D91D439EB42098F9550FBD9CF19",16) == new LargeInteger("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",36));
			Assertion.Assert("2r11 = 3r10", new LargeInteger("11",2) == new LargeInteger("10",3));
			
			LargeInteger l;
			try {
				l = new LargeInteger("1B93",11);
				Assertion.Assert("Invalid radix",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("--1",13);
				Assertion.Assert("--1",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("",7);
				Assertion.Assert("",false);
			} catch (ArgumentException) {
			}
			
			try {
				l = new LargeInteger("1 2",5);
				Assertion.Assert("1 2",false);
			} catch (ArgumentException) {
			}
		}
		
		[Test] public void UnsignedIntegerConstructor() {
			Assertion.Assert("12345",new LargeInteger(12345) == new LargeInteger(12345U));
		}
		
		[Test] public void LongConstructor() {
			Assertion.Assert("-98766",new LargeInteger(-98765L) == new LargeInteger(-98765));
			Assertion.Assert("-1<<63",new LargeInteger(-1L << 63) == ((-LargeInteger.One) << 63));
		}
		
		[Test] public void ULongConstructor() {
			Assertion.Assert("0x8765432112345678",new LargeInteger(8765432112345678UL) == new LargeInteger(8765432112345678L));
		}
		
		[Test] public void FloatConstructor() {
			Assertion.Assert("0.3 = 0",new LargeInteger(0) == new LargeInteger(0.3F));
			Assertion.Assert("12345678F = 12345678",new LargeInteger(12345678F) == new LargeInteger(12345678UL));
			Assertion.Assert("-12345678F = -12345678",new LargeInteger(12345678F) == new LargeInteger(12345678L));
			
			LargeInteger l;
			try {
				l = new LargeInteger(Single.NegativeInfinity);
				Assertion.Assert("+infinity", false);
			} catch (OverflowException) {}
			
			try {
				l = new LargeInteger(Single.PositiveInfinity);
				Assertion.Assert("-infinity", false);
			} catch (OverflowException) {}
			
			try {
				l = new LargeInteger(Single.NaN);
				Assertion.Assert("NaN", false);
			} catch (ArgumentException) {}
			
		}
		
		[Test] public void DoubleConstructor() {
			Assertion.Assert("0.3 = 0",new LargeInteger(0) == new LargeInteger(0.3));
			Assertion.Assert("1.23456789E10 = 12345678900",new LargeInteger(1.23456789E10) == new LargeInteger("12345678900"));
			Assertion.Assert("-1.23456789E10 = -12345678900",new LargeInteger(-1.23456789E10) == new LargeInteger("-12345678900"));
			
			LargeInteger l;
			try {
				l = new LargeInteger(Double.NegativeInfinity);
				Assertion.Assert("+infinity", false);
			} catch (OverflowException) {}
			
			try {
				l = new LargeInteger(Double.PositiveInfinity);
				Assertion.Assert("-infinity", false);
			} catch (OverflowException) {}
			
			try {
				l = new LargeInteger(Double.NaN);
				Assertion.Assert("NaN", false);
			} catch (ArgumentException) {}
			
		}
		
		[Test] public void UInt32ArrayConstructor() {
			uint[] bytes = {0x44442222U, 0xFFFF8888U};
			Assertion.Assert("0xFFFF888844442222 = 0x44442222,0xFFFF8888",
					new LargeInteger(0xFFFF888844442222UL) == new LargeInteger(bytes,false));
			Assertion.Assert("-0xFFFF888844442222 = -{0x44442222,0xFFFF8888}",
					new LargeInteger("-FFFF888844442222",16) == new LargeInteger(bytes,true));
			
			bytes = new uint[0];
			Assertion.Assert("!{}.Negative",!((new LargeInteger(bytes,true)).Negative));
		}
		
		[Test] public void ByteArrayConstructor() {
			byte[] bytes = {0xFF, 0x00, 0x88, 0x11, 0xEE, 0x22, 0x44, 0x55};
			Assertion.Assert("{0xFF, 0x00, 0x88, 0x11, 0xEE, 0x22, 0x44, 0x55}",
			                 new LargeInteger(bytes,false) == new LargeInteger(0x554422EE118800FFUL));
			Assertion.Assert("{0xFF, 0x00, 0x88, 0x11, 0xEE, 0x22, 0x44, 0x55}",
			                 new LargeInteger(bytes,true) == -(new LargeInteger(0x554422EE118800FFUL)));
			
			byte[] bytes1 = {0xFF,  0x00, 0x88, 0x11, 0xEE};
			Assertion.Assert("{0xFF, 0x00, 0x88, 0x11, 0xEE}",
			                 new LargeInteger(bytes1,false) == new LargeInteger(0xEE118800FFUL));
		}
		
		[Test] public void DecimalConstructor() {
			Assertion.Assert("0",(new LargeInteger(Decimal.Zero)) == LargeInteger.Zero);
			Assertion.Assert("1",(new LargeInteger(new Decimal(1))) == LargeInteger.One);
			Assertion.Assert("-1",(new LargeInteger(new Decimal(-1))) == LargeInteger.One.Negated());
			Assertion.Assert("2^32",(new LargeInteger(new Decimal(0x100000000))) == (LargeInteger.One << 32));
			Assertion.Assert("1.5",(new LargeInteger(new Decimal(1.5))) == LargeInteger.One);
			Assertion.Assert("-1.5",(new LargeInteger(new Decimal(-1.5))) == LargeInteger.One.Negated());
			Assertion.Assert("2^96 -1",(new LargeInteger(new Decimal(-1,-1,-1,false,0))) == ((LargeInteger.One << 96) - LargeInteger.One));
			Assertion.Assert("-2^96 +1",(new LargeInteger(new Decimal(-1,-1,-1,true,0))) == ((LargeInteger.One.Negated() << 96) + LargeInteger.One));
		}
		
		[Test] public void Negated() {
			LargeInteger num = new LargeInteger(12345);
			LargeInteger neg = new LargeInteger(-12345);
			
			Assertion.Assert("Negated", num == neg.Negated());
			Assertion.Assert("-",num == -neg);
			Assertion.Assert("-Negated", num == -num.Negated());
			Assertion.Assert("0 = -0", LargeInteger.Zero == -LargeInteger.Zero);
		}
		
		[Test] public void Adding() {
			Assertion.Assert("1 + 0", LargeInteger.One + LargeInteger.Zero == LargeInteger.One);
			Assertion.Assert("0 + 0", LargeInteger.Zero + LargeInteger.Zero == LargeInteger.Zero);
			Assertion.Assert("0 + 1", LargeInteger.Zero + LargeInteger.One == LargeInteger.One);
			
			double d = Math.Pow(2.0,400.0);
			LargeInteger l1 = new LargeInteger(d * 2.0);
			LargeInteger l2 = new LargeInteger(d);
			
			LargeInteger result = l1 + l2;
			Assertion.Assert("NonNegative1",result.NonNegative);
			Assertion.Assert("2**401 + 2**400",result == (new LargeInteger(d * 3.0)));
			
			result = -l1 + -l2;
			Assertion.Assert("negative1",result.NonPositive);
			Assertion.Assert("-2**401 + -2**400",result == (new LargeInteger(-d * 3.0)));
			
			result = -l1 + l2;
			Assertion.Assert("negative2",result.NonPositive);
			Assertion.Assert("-2**401 + 2**400",result == (new LargeInteger(-d)));
			
			result = l1 + -l2;
			Assertion.Assert("NonNegative2",result.NonNegative);
			Assertion.Assert("2**401 + -2**400",result == (new LargeInteger(d)));
			
			result = -l2 + l1;
			Assertion.Assert("NonNegative3",result.NonNegative);
			Assertion.Assert("-2**400 + 2**401",result == (new LargeInteger(d)));
			
			result = l1 + -l1;
			Assertion.Assert("NonNegative & negative",result.NonNegative && result.NonPositive);
			Assertion.Assert("2**401 + -2**401",result == (new LargeInteger(0)));
			
			Assertion.Assert("0 + NonNegative", LargeInteger.Zero + l1 == l1);
			Assertion.Assert("negative + 0", (-(LargeInteger.One << 40)) + LargeInteger.Zero == -(LargeInteger.One << 40));
			Assertion.Assert("0 + negative", LargeInteger.Zero + (-(LargeInteger.One << 40))== -(LargeInteger.One << 40));
		}
		
		[Test] public void Subtracting() {
			Assertion.Assert("1 - 0", LargeInteger.One - LargeInteger.Zero == LargeInteger.One);
			Assertion.Assert("0 - 0", LargeInteger.Zero - LargeInteger.Zero == LargeInteger.Zero);
			Assertion.Assert("0 - 1", LargeInteger.Zero - LargeInteger.One == -LargeInteger.One);
			
			double d = Math.Pow(2.0,400.0);
			LargeInteger l1 = new LargeInteger(d * 2.0);
			LargeInteger l2 = new LargeInteger(d);
			LargeInteger result = l1 - l2;
			
			Assertion.Assert("NonNegative1",result.NonNegative);
			Assertion.Assert("2**401 - 2**400",result == (new LargeInteger(d)));
			
			result = -l1 - -l2;
			Assertion.Assert("negative2",result.NonPositive);
			Assertion.Assert("-2**401 - -2**400",result == (new LargeInteger(-d)));
			
			result = -l1 - l2;
			Assertion.Assert("negative2",result.NonPositive);
			Assertion.Assert("-2**401 - 2**400",result == (new LargeInteger(-d * 3.0)));
			
			result = l1 - -l2;
			Assertion.Assert("NonNegative2",result.NonNegative);
			Assertion.Assert("2**401 - -2**400",result == (new LargeInteger(d * 3.0)));
			
			result = l1 - l1;
			Assertion.Assert("NonNegative & negative",result.NonNegative && result.NonPositive);
			Assertion.Assert("2**401 - 2**401",result == (new LargeInteger(0)));
			
			Assertion.Assert("num - 0", l1 - LargeInteger.Zero == l1);
			Assertion.Assert("0 - num", LargeInteger.Zero - l1 == l1.Negated());
			Assertion.Assert("0 - num.Negated()", LargeInteger.Zero - l1.Negated() == l1);
		}
		
		[Test] public void BitShifting() {
			double d = Math.Pow(2.0,320);
			LargeInteger l = new LargeInteger(d);
			
			Assertion.Assert("0<<10000", LargeInteger.Zero << 10000 == LargeInteger.Zero);
			Assertion.Assert("2**320 == 1<<320",l == (LargeInteger.One << 320));
			Assertion.Assert("3*(2**320) == 3<<320",new LargeInteger(3*d) == ((new LargeInteger(3)) << 320));
			
			Assertion.Assert("2**320 >> 320 = 1",l >> 320 == LargeInteger.One);
			Assertion.Assert("2**320 >> 321 = 0",l >> 321 == LargeInteger.Zero);
			
			d = 1;
			l = LargeInteger.One;
			for (int i = 0;i<100;i++) {
				d *= 2;
				l <<= 1;
				Assertion.Assert(i.ToString(),l == new LargeInteger(d));
			}
			
			for (int i = 0;i<100;i++) {
				d /= 2;
				l >>= 1;
				Assertion.Assert((-i).ToString(),l == new LargeInteger(d));
			}
			
			Assertion.Assert("-1>>1",!((-LargeInteger.One) >> 1).Negative);
			Assertion.Assert("-1<<1",((-LargeInteger.One) << 1).Negative);
			
			Assertion.Assert("-(100!)",(-(LargeInteger.Factorial(100))).BitShift(-10).Negative);
		    
		    Assertion.Assert("0 >> 100 IsInt32()", (LargeInteger.Zero >> 100).IsInt32());
		}
		
		[Test] public void BitAnd() {
			Assertion.Assert("0xFFFF 0x0000",((new LargeInteger(0xFFFF)) & LargeInteger.Zero) == LargeInteger.Zero);
			Assertion.Assert("0xFFFF 0xF00F",
			                 ((new LargeInteger(0xFFFF)) & (new LargeInteger(0xF00F))) == (new LargeInteger(0xF00F)));
			Assertion.Assert("0xFFFF 0xFFFFFFFF00000001",
			                 ((new LargeInteger(0xFFFF)) & (new LargeInteger(0xFFFFFFFF00000001UL))) == LargeInteger.One);
		}
		
		[Test] public void BitOr() {
			LargeInteger l;
			
			l = new LargeInteger(0xFFFF);
			Assertion.Assert("0xFFFF 0x0000",(l | LargeInteger.Zero) == l);
			Assertion.Assert("0xFFFF 0xF00F",(l | (new LargeInteger(0xF00F))) == l);
			Assertion.Assert("0xFFFF 0xFFFFFFFF00000001",
			                 (l | (new LargeInteger(0xFFFFFFFF00000001UL))) == (new LargeInteger(0xFFFFFFFF0000FFFFUL)));
		}
		
		[Test] public void BitXor() {
			LargeInteger l;
			
			l = new LargeInteger(0xFFFF);
			Assertion.Assert("0xFFFF 0x0000",(l ^ LargeInteger.Zero) == l);
			Assertion.Assert("0xFFFF 0xF00F",(l ^ (new LargeInteger(0xF00F))) == (new LargeInteger(0x0FF0)));
			Assertion.Assert("0xFFFF 0xFFFFFFFF00000001",
			                 (l ^ (new LargeInteger(0xFFFFFFFF00000001UL))) == (new LargeInteger(0xFFFFFFFF0000FFFEUL)));
			
			l = LargeInteger.Factorial(40);
			Assertion.Assert("40! 40!",l.BitXor(l) == LargeInteger.Zero);
		}
		
		[Test] public void Multiply() {
			LargeInteger l = new LargeInteger(1234567890U);
			
			Assertion.Assert("* 0",l * LargeInteger.Zero == LargeInteger.Zero);
			Assertion.Assert("0 *",LargeInteger.Zero * l == LargeInteger.Zero);
			Assertion.Assert("* 1",l * LargeInteger.One == l);
			Assertion.Assert("1 *",LargeInteger.One * l == l);
			Assertion.Assert("10 *",(new LargeInteger(10) * l) == (new LargeInteger(12345678900UL)));
			
			Assertion.Assert("* -1",l * -LargeInteger.One == -l);
			Assertion.Assert("- * -1",-l * -LargeInteger.One == l);
			Assertion.Assert("- * 1",-l * LargeInteger.One == -l);
			
			Assertion.Assert("10 Factorial",LargeInteger.Factorial(10) == new LargeInteger(3628800));
			
			Assertion.Assert(new LargeInteger("2216980800") * new LargeInteger("1562389080") *
			                 new LargeInteger("1072431360") * new LargeInteger("713897640") *
			                 new LargeInteger("458377920") * new LargeInteger("281887200") *
			                 new LargeInteger("164490480") * new LargeInteger("89927760") *
			                 new LargeInteger("45239040") * new LargeInteger("20389320") *
			                 new LargeInteger("7893600") * new LargeInteger("2441880") *
			                 new LargeInteger("524160") * new LargeInteger("39916800") ==
			                 	new LargeInteger("1885494701666050254987932260861146558230394535379329335672487982961844043495537923117729972224000000000000000000"));
		}
		
		[Test] public void Divide() {
			Assertion.Assert("100 Factorial / 99 Factorial",LargeInteger.Factorial(100) / LargeInteger.Factorial(99) == new LargeInteger(100));
			Assertion.Assert("100 Factorial / 100",LargeInteger.Factorial(100) / new LargeInteger(100) ==  LargeInteger.Factorial(99));
			Assertion.Assert("1 / 50 Factorial",LargeInteger.One / LargeInteger.Factorial(50) == LargeInteger.Zero);
			Assertion.Assert("20 Factorial / 20 Factorial",LargeInteger.Factorial(20) / LargeInteger.Factorial(20) == LargeInteger.One);
			Assertion.Assert("20! / 21!",LargeInteger.Factorial(20) / LargeInteger.Factorial(21) == LargeInteger.Zero);
			Assertion.Assert("add back 0x10000000000000000000000000000 / 0x8000000000000000FFFFFFFF",
			                 (new LargeInteger("10000000000000000000000000000",16)) / 
			                 (new LargeInteger("8000000000000000FFFFFFFF",16)) == (new LargeInteger(0x1FFFF)));
			
			Assertion.Assert("100! / -99!",LargeInteger.Factorial(100) / -LargeInteger.Factorial(99) == -(new LargeInteger(100)));
			Assertion.Assert("-100! / 99!",-LargeInteger.Factorial(100) / LargeInteger.Factorial(99) == -(new LargeInteger(100)));
			Assertion.Assert("-100! / -99!",-LargeInteger.Factorial(100) / -LargeInteger.Factorial(99) == (new LargeInteger(100)));
			
			Assertion.Assert("0x3FFFFFFFC00000000000000000000000 / 0x40000000FFFFFFFF",
			                 (new LargeInteger("3FFFFFFFC00000000000000000000000",16)) / 
			                 (new LargeInteger("40000000FFFFFFFF",16)) == 
			                 	(new LargeInteger("FFFFFFFB00000017",16)));
			Assertion.Assert("UInt", (new LargeInteger("3FFFFFFFC00000000000000000000000",16)) /
			                 (new LargeInteger(0x80000001U)) == new LargeInteger("39614081229462052699092549626"));
			
			LargeInteger four = new LargeInteger(4);
			LargeInteger three = new LargeInteger(3);
			Assertion.Assert("-4 / 3", -four / three == new LargeInteger(-4 / 3));
			Assertion.Assert("-4 / -3", -four / -three == new LargeInteger(-4 / -3));
			Assertion.Assert("4 / -3", four / -three == new LargeInteger(4 / -3));
		}
		
		[Test] public void LessThan() {
			LargeInteger l = new LargeInteger(0xFFFF0000FFFF0000UL);
			
			Assertion.Assert("!(1 < 0)",!(LargeInteger.One < LargeInteger.Zero));
			Assertion.Assert("0 < 1",LargeInteger.Zero < LargeInteger.One);
			Assertion.Assert("-1 < 0",-LargeInteger.One < LargeInteger.Zero);
			Assertion.Assert("!(l < 1)",!(l < LargeInteger.One));
			Assertion.Assert("-l < 1",-l < LargeInteger.One);
			Assertion.Assert("!(l < l)",!(l < l));
			Assertion.Assert("!(l < -l)",!(l < -l));
		}
		
		[Test] public void LessThanEqualTo() {
			LargeInteger l = new LargeInteger(0xFFFF0000FFFF0000UL);
			
			Assertion.Assert("!(1 <= 0)",!(LargeInteger.One <= LargeInteger.Zero));
			Assertion.Assert("0 <= 1",LargeInteger.Zero <= LargeInteger.One);
			Assertion.Assert("-1 <= 0",-LargeInteger.One <= LargeInteger.Zero);
			Assertion.Assert("!(l <= 1)",!(l <= LargeInteger.One));
			Assertion.Assert("-l <= 1",-l <= LargeInteger.One);
			Assertion.Assert("l <= l",l <= l);
			Assertion.Assert("!(l <= -l)",!(l <= -l));
		}
		
		[Test] public void GreaterThan() {
			LargeInteger l = new LargeInteger(0xFFFF0000FFFF0000UL);
			
			Assertion.Assert("1 > 0",LargeInteger.One > LargeInteger.Zero);
			Assertion.Assert("!(0 > 1)",!(LargeInteger.Zero > LargeInteger.One));
			Assertion.Assert("!(-1 > 0)",!(-LargeInteger.One > LargeInteger.Zero));
			Assertion.Assert("l > 1",l > LargeInteger.One);
			Assertion.Assert("!(-l > 1)",!(-l > LargeInteger.One));
			Assertion.Assert("!(l > l)",!(l > l));
			Assertion.Assert("l > -l",l > -l);
		}
		
		[Test] public void GreaterThanEqualTo() {
			LargeInteger l = new LargeInteger(0xFFFF0000FFFF0000UL);
			
			Assertion.Assert("1 >= 0",LargeInteger.One >= LargeInteger.Zero);
			Assertion.Assert("!(0 >= 1)",!(LargeInteger.Zero >= LargeInteger.One));
			Assertion.Assert("!(-1 >= 0)",!(-LargeInteger.One >= LargeInteger.Zero));
			Assertion.Assert("l >= 1",l >= LargeInteger.One);
			Assertion.Assert("!(-l >= 1)",!(-l >= LargeInteger.One));
			Assertion.Assert("l >= l",l >= l);
			Assertion.Assert("l >= -l",l >= -l);
		}
		
		[Test] public void Mod() {
			Assertion.Assert("100 Factorial % 99 Factorial",LargeInteger.Factorial(100) % LargeInteger.Factorial(99) == LargeInteger.Zero);
			Assertion.Assert("100 Factorial % 100",LargeInteger.Factorial(100) % new LargeInteger(100) ==  LargeInteger.Zero);
			Assertion.Assert("1 % 50 Factorial",LargeInteger.One % LargeInteger.Factorial(50) == LargeInteger.One);
			Assertion.Assert("20 Factorial % 20 Factorial",LargeInteger.Factorial(20) % LargeInteger.Factorial(20) == LargeInteger.Zero);
			Assertion.Assert("20! % 21!",LargeInteger.Factorial(20) % LargeInteger.Factorial(21) == LargeInteger.Factorial(20));
			Assertion.Assert("0x1000000000000000000000000 % 0x100000000000FFFFFFFF",(new LargeInteger("1000000000000000000000000",16)) % (new LargeInteger("100000000000FFFFFFFF",16)) == (new LargeInteger("FFFFFF00001000FFFFF",16)));
			
			Assertion.Assert("100! + 1 % -99!",(LargeInteger.Factorial(100) + LargeInteger.One) % -LargeInteger.Factorial(99) == LargeInteger.One);
			Assertion.Assert("-100! - 1 % 99!",(-LargeInteger.Factorial(100) - LargeInteger.One) % LargeInteger.Factorial(99) == -LargeInteger.One);
			Assertion.Assert("-100! - 1 % -99!",(-LargeInteger.Factorial(100) - LargeInteger.One) % -LargeInteger.Factorial(99) == -LargeInteger.One);
			
			Assertion.Assert("1<<20321573 - 1 % 4997061240154980247 = 0",((LargeInteger.One << 20321573) - LargeInteger.One) % new LargeInteger("4997061240154980247") == LargeInteger.Zero);
			Assertion.Assert("UInt", (new LargeInteger("3FFFFFFFC00000000000000000000000",16)) %
			                 (new LargeInteger(0x80000001U)) == new LargeInteger(6));
			
			LargeInteger four = new LargeInteger(4);
			LargeInteger three = new LargeInteger(3);
			Assertion.Assert("-4 % 3", -four % three == new LargeInteger(-4 % 3));
			Assertion.Assert("-4 % -3", -four % -three == new LargeInteger(-4 % -3));
			Assertion.Assert("4 % -3", four % -three == new LargeInteger(4 % -3));
		}
		
		[Test] public void ToStringTest() {
			LargeInteger l = LargeInteger.Factorial(30);
			
			Assertion.Assert((new LargeInteger(l.ToString())) == l);
			Assertion.Assert((new LargeInteger((-LargeInteger.One).ToString())) == -LargeInteger.One);
			Assertion.Assert((new LargeInteger(LargeInteger.Zero.ToString())) == LargeInteger.Zero);
			Assertion.Assert("100!",LargeInteger.Factorial(100).ToString() == "93326215443944152681699238856266700490715968264381621468592963895217599993229915608941463976156518286253697920827223758251185210916864000000000000000000000000");
		}
		
		[Test] public void ToStringRadixTest() {
			LargeInteger l = LargeInteger.Factorial(35);
			
			for (int i = 2; i <= 36; i++) {
				Assertion.Assert(i.ToString(), l == new LargeInteger(l.ToString(i),i));
			}
		}
		
		[Test] public void PowerTest() {
			Assertion.Assert("1^1000",LargeInteger.One.Pow(1000) == LargeInteger.One);
			Assertion.Assert("2^1000",(LargeInteger.One + LargeInteger.One).Pow(1000) == LargeInteger.One.BitShift(1000));
			Assertion.Assert("-1^30",(-LargeInteger.One).Pow(30) == LargeInteger.One);
			Assertion.Assert("-1^31",(-LargeInteger.One).Pow(31) == -LargeInteger.One);
			Assertion.Assert("100^0",(new LargeInteger(100)).Pow(0) == LargeInteger.One);
			Assertion.Assert("-2^5001<0", (new LargeInteger(-2)).Pow(5001).Negative);
		}
		
		[Test] public void PowerModTest() {
			Assertion.Assert("1^1000%10",
			                 LargeInteger.One.PowMod(new LargeInteger(1000), new LargeInteger(10)) == LargeInteger.One);
			Assertion.Assert("13^2%11", 
			                 (new LargeInteger(13)).PowMod(new LargeInteger(2), new LargeInteger(11)) == 
			                 	new LargeInteger(4));
			Assertion.Assert("13^(3^1024)%2995680650762294161",
			                 (new LargeInteger(13)).PowMod((new LargeInteger(3)).Pow(1024), new LargeInteger("2995680650762294161")) ==
			                 	new LargeInteger("2285829289481565747"));
		}
		
		[Test] public void ToDoubleTest() {
			Assertion.Assert("-1",(-LargeInteger.One).ToDouble() == -1.0);
			Assertion.Assert("2^500",(new LargeInteger(2)).Pow(500).ToDouble() == Math.Pow(2.0,500.0));
			Assertion.Assert("2^5000",Double.IsPositiveInfinity((new LargeInteger(2)).Pow(5000).ToDouble()));
			Assertion.Assert("-2^5001",Double.IsNegativeInfinity((new LargeInteger(-2)).Pow(5001).ToDouble()));
			
			ulong number = 0;
			for (int i = 0; i < 64; i++) {
				Assertion.Assert(i.ToString(),(new LargeInteger(number)).ToDouble() == (double)number);
				number <<= 1;
				number++;
			}
		}
		
		[Test] public void ToFloatTest() {
			Assertion.Assert("-1",(-LargeInteger.One).ToFloat() == -1.0F);
			Assertion.Assert("2^50",(new LargeInteger(2)).Pow(50).ToFloat() == (float)Math.Pow(2.0,50.0));
			Assertion.Assert("2^500",Double.IsPositiveInfinity((new LargeInteger(2)).Pow(500).ToFloat()));
			Assertion.Assert("-2^501",Double.IsNegativeInfinity((new LargeInteger(-2)).Pow(501).ToFloat()));
			
			ulong number = 0;
			for (int i = 0; i < 64; i++) {
				Assertion.Assert(i.ToString(),(new LargeInteger(number)).ToFloat() == (float)number);
				number <<= 1;
				number++;
			}
		}
		
		[Test] public void BitCountTest() {
			Assertion.Assert("0",LargeInteger.Zero.BitCount == 0);
			Assertion.Assert("1",LargeInteger.One.BitCount == 1);
			Assertion.Assert("-1",(-LargeInteger.One).BitCount == 1);
			Assertion.Assert("0x1FFFFFFFF",(new LargeInteger(0x1FFFFFFFF)).BitCount == 33);
			Assertion.Assert("0xFFFFFFFF",(new LargeInteger(0xFFFFFFFF)).BitCount == 32);
		}
		
		[Test] public void BitTestingTest() {
			ulong number = 0x123456789ABCDEF;
			LargeInteger large = new LargeInteger(number);
			
			for (int i = 0; i < 64; i++) {
				Assertion.Assert(number.ToString(),large[i] == ((number & 0x1) == 0x1));
				number >>= 1;
			}
		}
		
		[Test] public void AbsTest() {
			Assertion.Assert("0",LargeInteger.Zero.Abs() == LargeInteger.Zero);
			Assertion.Assert("1",LargeInteger.One.Abs() == LargeInteger.One);
			Assertion.Assert("-1",(-LargeInteger.One).Abs() == LargeInteger.One);
		}
		
		[Test] public void GcdTest() {
			Assertion.Assert("P1.GCD(P2)",(new LargeInteger(4997061240154980247)).Gcd(new LargeInteger(14174928510895163201)) == LargeInteger.One);
			Assertion.Assert("1FFFFFFFFFFF7 gcd 893473623423", (new LargeInteger(0x1FFFFFFFFFFF7)).Gcd(new LargeInteger(0x893473623423)) == new LargeInteger(7));
		}
		
		[Test] public void EqualsTest() {
			Assertion.Assert("(1 << 400).Equals(2^400)",
			                 Compare(LargeInteger.One << 400,(new LargeInteger(2)).Pow(400)));
			Assertion.Assert("!(1 << 400 + 1).Equals(2^400)",
			                 !Compare((LargeInteger.One << 400) + LargeInteger.One,(new LargeInteger(2)).Pow(400)));
			
			Assertion.Assert("Different types",!Compare(this,LargeInteger.One));
		}
		
		protected bool Compare(object l1, object l2) {
			if (l1.Equals(l2) != l2.Equals(l1)) throw new ApplicationException("Equals different");
			if (l1.Equals(l2)) {
				if (l1.GetHashCode() != l2.GetHashCode()) throw new ApplicationException("Hash different");
			}
			
			return l1.Equals(l2);
		}
		
		[Test] public void ToIntTest() {
			Assertion.Assert("0's test", 
			                 LargeInteger.Zero.IsInt32() &&
			                 LargeInteger.Zero.IsInt64() &&
			                 LargeInteger.Zero.IsUInt32() &&
			                 LargeInteger.Zero.IsUInt64());
			Assertion.Assert("0's to int", 
			                 LargeInteger.Zero.ToInt32() == 0 &&
			                 LargeInteger.Zero.ToInt64() == 0 &&
			                 LargeInteger.Zero.ToUInt32() == 0 &&
			                 LargeInteger.Zero.ToUInt64() == 0);
			Assertion.Assert("1's test", 
			                 LargeInteger.One.IsInt32() &&
			                 LargeInteger.One.IsInt64() &&
			                 LargeInteger.One.IsUInt32() &&
			                 LargeInteger.One.IsUInt64());
			Assertion.Assert("1's to int", 
			                 LargeInteger.One.ToInt32() == 1 &&
			                 LargeInteger.One.ToInt64() == 1 &&
			                 LargeInteger.One.ToUInt32() == 1 &&
			                 LargeInteger.One.ToUInt64() == 1);
			Assertion.Assert("-1's test", 
			                 (-LargeInteger.One).IsInt32() &&
			                 (-LargeInteger.One).IsInt64() &&
			                 (-LargeInteger.One).IsUInt32() &&
			                 (-LargeInteger.One).IsUInt64());
			Assertion.Assert("-1's to int", 
			                 (-LargeInteger.One).ToInt32() == -1 &&
			                 (-LargeInteger.One).ToInt64() == -1 &&
			                 (-LargeInteger.One).ToUInt32() == 1 &&
			                 (-LargeInteger.One).ToUInt64() == 1);
			
			LargeInteger l = new LargeInteger(0x80000000);
			Assertion.Assert("0x80000000 test",
			                 !l.IsInt32() && l.IsUInt32() && l.IsInt64() && l.IsUInt64());
			Assertion.Assert("0x80000000 to int",
			                 l.ToUInt32() == 0x80000000 &&
			                 l.ToInt64() == 0x80000000L &&
			                 l.ToUInt64() == 0x80000000UL);
			try {
				l.ToInt32();
				Assertion.Assert("0x80000000 to ToInt32", false);
			} catch (OverflowException) {}
			
			l = -l;
			Assertion.Assert("-0x80000000 test",
			                 l.IsInt32() && l.IsUInt32() && l.IsInt64() && l.IsUInt64());
			Assertion.Assert("0x80000000 to int",
			                 l.ToInt32() == -0x80000000 && 
			                 l.ToUInt32() == 0x80000000 &&
			                 l.ToInt64() == -0x80000000L &&
			                 l.ToUInt64() == 0x80000000UL);
			
			l -= LargeInteger.One;
			Assertion.Assert("-0x80000001 is int", !l.IsInt32());
			try {
				l.ToInt32();
				Assertion.Assert("-0x80000001 to int 32", false);
			} catch (OverflowException) {}
			Assertion.Assert("-0x80000001 to uint", l.ToUInt32() == 0x80000001);
			
			l = new LargeInteger(0x8000000000000000);
			Assertion.Assert("0x8000000000000000 test",
			                 !l.IsInt32() && !l.IsUInt32() && !l.IsInt64() && l.IsUInt64());
			Assertion.Assert("0x8000000000000000 to int", l.ToUInt64() == 0x8000000000000000UL);
			try {
				l.ToInt64();
				Assertion.Assert("0x8000000000000000 to int64", false);
			} catch (OverflowException) {}
			try {
				l.ToUInt32();
				Assertion.Assert("0x8000000000000000 to uint32", false);
			} catch (OverflowException) {}
			
			l = -l;
			Assertion.Assert("-0x8000000000000000 test",
			                 !l.IsInt32() && !l.IsUInt32() && l.IsInt64() && l.IsUInt64());
			Assertion.Assert("-0x8000000000000000 to int",
			                 l.ToInt64() == Int64.MinValue &&
			                 l.ToUInt64() == 0x8000000000000000);
			
			l -= LargeInteger.One;
			Assertion.Assert("-0x8000000000000001 test",
			                 !l.IsInt32() && !l.IsUInt32() && !l.IsInt64() && l.IsUInt64());
			try {
				l.ToInt64();
				Assertion.Assert("Int64.MinValue - 1", false);
			} catch (OverflowException) {}
			
			l = new LargeInteger(UInt64.MaxValue);
			Assertion.Assert("UInt64 MaxValue test", l.IsUInt64());
			Assertion.Assert("UInt64 MaxValue to int", l.ToUInt64() == UInt64.MaxValue);
			
			l += LargeInteger.One;
			Assertion.Assert("UInt64.MaxValue + 1 test", !l.IsUInt64());
			try {
				l.ToUInt64();
				Assertion.Assert("UInt64.MaxValue + 1", false);
			} catch (OverflowException) {}
		}
		
		[Test] public void ToDecimalTest() {
			Assertion.Assert("1",LargeInteger.One.ToDecimal() == new Decimal(1));
			Assertion.Assert("-1",LargeInteger.One.Negated().ToDecimal() == new Decimal(-1));
			Assertion.Assert("0",LargeInteger.Zero.ToDecimal() == new Decimal(0));
			Assertion.Assert("2^32-1",((LargeInteger.One << 32) - LargeInteger.One).ToDecimal() == new Decimal(0xFFFFFFFF));
			Assertion.Assert("-2^32+1",(LargeInteger.One << 31).ToDecimal() == new Decimal(0x80000000));
			Assertion.Assert("-2^96+1",((LargeInteger.One << 96) - LargeInteger.One).Negated().ToDecimal() == new Decimal(-1,-1,-1,true,0));
			Assertion.Assert("!2^96 isDecimal",!(LargeInteger.One << 96).IsDecimal());
			Assertion.Assert("2^96-1 isDecimal",((LargeInteger.One << 96) - LargeInteger.One).IsDecimal());
			Assertion.Assert("0 isDecimal",LargeInteger.Zero.IsDecimal());
		}
	}

}	
